//
//  ViewModel.swift
//  test-activity-indicator
//
//  Created by Huy Trinh Duc on 15/05/2023.
//

import Foundation
import Combine
import CombineExt
import ActivityIndicator

class ViewModel: ObservableObject {
    
    @Published var username = ""
    @Published var password = ""
    
    @Published var isEnableLoginButton = false
    
    let triggerLoginSubject = PassthroughSubject<Void, Never>()
    
    let api = APIService()
    
    let errorIndicator = ErrorIndicator()
    
    private var bag = Set<AnyCancellable>()

    /// Recommend to expose the errors only
    var errorPublisher: AnyPublisher<Error, Never> {
        errorIndicator.errors.eraseToAnyPublisher()
    }
    
    @Published var errorCatchPublisher: Error? = nil
    
    init() {
        
        isEnableLoginButtonPublisher
            .assign(to: &$isEnableLoginButton)
        
        let credentials = Publishers.CombineLatest($username, $password)
        
        triggerLoginSubject
            .withLatestFrom(credentials)
            .flatMap { [self] _, _ in
                api.getTestApiPublisher()
                    .receive(on: RunLoop.main)
//                    .trackError(errorIndicator)
                    .catch { error -> AnyPublisher<TestModel, Never> in
                        self.errorCatchPublisher = error
                        return Empty().eraseToAnyPublisher()
                    }
            }
            .sink { model in
                print(model.title)
            }
            .store(in: &bag)
    }
    
    func triggerLogin() {
        triggerLoginSubject
            .send(())
    }
    
    private var isEnableLoginButtonPublisher: AnyPublisher<Bool, Never> {
        Publishers.CombineLatest($username, $password)
            .map({!$0.isEmpty && !$1.isEmpty})
            .eraseToAnyPublisher()
    }
    
}
